


%% calculates the matrices for REE solution. This is the version with
%% heterogeneous elasticity of substitutions and cost push shocks


function [A, B, C, kAlphaR, kAlphaC, kAlphaZ, dimK, W_mat] =...
    param_natural_target6_cs(N, sigma,  beta, gamma, theta_vec, eta,...
    alpha_vec, lambda_vec, zetaZAlpha, opt_weights,an_vec,rho_cts);

%disp('start')
%global A B C W_mat

%% NOTE: 1. opt_weights defines a vector of size N including the 
%%          inflation and output optimal weights                                              
%%       2. an_vec defines the expenditure shares from the model
%%       3. lambda_vec defines the curvature in the sector-specific
%%          production function
%%       4. the TARGETING RULE responds also to the consumption-gap.
%%       The Consumption variable in the model is consumption in deviations from its
%%       deterministic trend.

%% Comment if used as a function
%  clear all;%clc;
%  N = 3;
%  sigma  = 1;
%  beta =0.99;
%  gamma =1;
% % theta_vec = [8;5;13];
%  eta = 0.5;
% % alpha_vec = [0.6;0.8;0.1];
% % lambda_vec = [1.4;1.2;1.1];
%  rho_ss = 0.7;  %% autoregressive coefficient
% %                 %% NOTE: for now we assume that every sector has the same
% %                 %% rho
%  zetaZAlpha = rho_ss.*ones(N,1);   %% vector of autoregressive coeffcients
% % an_vec = [0.3;0.5;0.2];
% % opt_weights = [0;an_vec];
% % 
% % %trial from text
% % an_vec = [50; 50]; 
%  theta=5;
% an_vec = randn(N,1);
% an_vec = an_vec./sum(an_vec);
% %an_vec = [20; 80]; 
% 
% %an_vec = an_vec/100;
% 
% theta_vec = theta.*ones(N,1);  
% 
% an_vec(N) = 1-sum(an_vec(1:N-1));
% 
% alpha_vec = 66.*ones(N,1);
% 
% alpha_vec = alpha_vec./100;
% %alpha_vec = [0.7;0.1];
% lambda_vec  = ones(N,1);
% 
% rand_weights = abs(randn(N+1,1));
% 
%  opt_weights = rand_weights./sum(rand_weights);

%% Define Phillips curve coefficients

eta_vec = lambda_vec.*(1+eta)-1; %% convolution of parameters...

czAlpha = -(eta_vec+1)./eta_vec; %% shocks' coefficients


den_vec = 1.+(theta_vec.*eta_vec); %% this is  the vector 1+theta(n)*eta in the
                               %% Phillips curve. It expresses the common
                               %% denominator of the variables'
                               %% coefficients

el_mc_vec = ((1-alpha_vec.*beta).*(1-alpha_vec))...
      ./alpha_vec; %% defines vector of the elasticity of inflation to the
                    %%% mc in each sector
                                  
                    
kAlphaR = el_mc_vec.*((1+gamma.*eta_vec)./den_vec);  %% relative prices



kAlphaC = el_mc_vec.*((sigma+eta_vec)./den_vec); %% consumption



kAlphaZ = el_mc_vec.*((eta_vec.*czAlpha)./den_vec);  %% coefficient on sector shock


%% Consumption under flexible prices

ccf = -(sum(an_vec.*((sigma+eta_vec)./(1+eta_vec))))^(-1);

ccfAlpha_vec = ccf*((an_vec.*eta_vec.*czAlpha)./(1+eta_vec)); %% individual weight on sector-specific shocks

                                          


%% [1] Make A
dimA=3*N+6; %% NOTE: make sure this number is consistent with the list below 

%% LIST OF EQUATIONS

%% 1. targeting rule (cons)
%% 2. inflation (CPI)
%% 3. opt_weight inflation
%% 4. average shock
%% 5. Nth relative price gap
%% 6. Nominal interest rate
%% 7. N sector-specific inflation rates
%% 8. N-1 relative price gaps
%% 9. N sector-specific shocks
%% 10. common tech. shock

%General Red-Solds Parameters


            dimK=2*N-1+1; %number state variables (N sect-spec shocks, 
                          %N-1 relative prices and 1 common tech. shock
            

jump_agg = 6; %% number of aggregate jump variables before the sector 
              %% specific inflation.

              
A=zeros(dimA,dimA);

%% Targeting rule

% A(1,2*N-1+jump_agg+1:3*N-1+jump_agg) = opt_weights(1).*ccfAlpha_vec; %% consumption
%                                             %% under flexible prices (as a
%                                             %% function of sect. shocks)

%A(1,2*N-1+jump_agg+1:3*N-1+jump_agg) = zeros(size(ccfAlpha_vec,1),1); %% no weight on consumption under flex prices


%% Nth relative price gap


A(5,N+jump_agg+1:2*N-1+jump_agg) = -an_vec(1:N-1)./an_vec(N);


%% Nominal interest rate

A(6,1) = 1;  %% consumption

A(6,2) = sigma^(-1); %% CPI inflation


%% All Sector specific inflation rates

A(jump_agg+1:N+jump_agg, jump_agg+1:N+jump_agg)...
    =beta.*eye(N); %% future expected inflation rates

A(jump_agg+1:N+jump_agg, 2*N-1+jump_agg+1:3*N-1+jump_agg) = diag(kAlphaZ); 
                   %% sector-specific shocks
                   
A(jump_agg+1:N+jump_agg,3*N-1+jump_agg+1) = kAlphaZ; %% common shock

                   
%% N-1 Sector specific inflation


A(jump_agg+1:N+jump_agg-1, N+jump_agg+1:2*N-1+jump_agg)...
    =-diag(kAlphaR(1:N-1)); %% N-1 relative prices






%% N-1 Relative prices

A(N+jump_agg+1:2*N-1+jump_agg,N+jump_agg+1:2*N-1+jump_agg) = eye((N-1)); %% N-1 relative prices



%% sector-specific shocks

A(2*N-1+jump_agg+1:3*N-1+jump_agg,2*N-1+jump_agg+1:3*N-1+jump_agg) = eye(N);


%% common shock

A(3*N-1+jump_agg+1,3*N-1+jump_agg+1) = 1;

%% Weighted average of sector-specific shocks

 
A(4,2*N-1+jump_agg+1:3*N-1+jump_agg) = an_vec';






%% [2] Make B


dimB=dimA;

B=zeros(dimB,dimB);


%% Targeting rule

%% LIST OF EQUATIONS

%% 1. targeting rule (cons)
%% 2. inflation (CPI)
%% 3. opt_weight inflation
%% 4. average shock
%% 5. Nth relative price gap
%% 6. Nominal interest rate
%% 7. N sector-specific inflation rates
%% 8. N-1 relative price gaps
%% 9. N sector-specific shocks
%% 10. common tech. shock

%% Targeting rule

% B(1,1) = opt_weights(1);   %% consumption [NOT USED HERE: zero weitght on consumption gap]

B(1,jump_agg+1:N+jump_agg) = opt_weights(2:end)'; %% sectoral inflation rates


%% Inflation (CPI)

B(2,2) = 1;

B(2,jump_agg+1:N+jump_agg) = -an_vec';


%% opt weight index

B(3,3) = 1;

B(3,jump_agg+1:N+jump_agg) = -opt_weights(2:end)';


%% Weighted average of sector-specific shocks

B(4,4) = 1;

%% Nth Relative price 

B(5,5) = 1;


%% Nominal interest rate

B(6,6) = sigma^(-1); %% nominal interest rate

B(6,1) = 1;  %% consumption


%% All sector specific inflation rates


B(jump_agg+1:N+jump_agg,jump_agg+1:N+jump_agg) = eye(N); %% inflation rates


B(jump_agg+1:N+jump_agg,1) = -kAlphaC(1:N);  %% consumption 


%% Nth sector specific inflation (depends on the Nth relative price)

B(N+jump_agg,5) = kAlphaR(N); %% Nth relative price 







%% N-1 Relative prices

B(N+jump_agg+1:2*N-1+jump_agg,2) = -ones((N-1),1);  %% CPI inflation

B(N+jump_agg+1:2*N-1+jump_agg,jump_agg+1:(N-1)+jump_agg)...
                     = eye((N-1)); %% sec. spec. infl.

B(N+jump_agg+1:2*N-1+jump_agg,N+jump_agg+1:2*N-1+jump_agg)...
                    = eye((N-1));  %% lag relative price





%% Sector specific shocks

B(2*N-1+jump_agg+1:3*N-1+jump_agg,2*N-1+jump_agg+1:3*N-1+jump_agg)...
                   = diag(zetaZAlpha);  %% autocorrelation coeff.

B(3*N-1+jump_agg+1,3*N-1+jump_agg+1) = rho_cts;


%% [3] Make C
dimC = [dimA N+1];  %% sector-specific shocks (N) 

C = zeros(dimC);



C(2*N-1+jump_agg+1:3*N-1+jump_agg,1:N) = eye(N);


C(3*N-1+jump_agg+1,N+1) = 1;
                               
                                  
%% [4] Set the weight matrix

%% weights

% square terms
xi1 = sigma+sum(an_vec.*eta_vec); %% weight on consumption

xi2_vecN_1 = an_vec(1:N-1).*(1+eta_vec(1:N-1)); %% vector of weights on relative prices

xi_2_N = an_vec(N)*(1+eta_vec(N)); %% weight on the last relative price

xi_infl = el_mc_vec./den_vec; 

xi3_vec = ((an_vec.*theta_vec.*(xi_infl.^(-1)))); %% vector of weight on sector
                                                  %% specific inflation
  
% cross terms

% relative prices*consumption (N-1 relaive prices)

xi_rpc_vec = an_vec(1:N-1).*eta_vec(1:N-1);

% relative prices*sector shocks (N-1 relative prices)

xi_rpz_vec = an_vec(1:N-1).*eta_vec(1:N-1).*czAlpha(1:N-1);


% sector shocks*consumption

xi_cz_vec =an_vec.*eta_vec.*czAlpha;


%% W matrix

W_mat = zeros(3*N-1+jump_agg+1,3*N-1+jump_agg+1);

W_mat(1,1) = xi1;  %% consumption

W_mat(5,5) = xi_2_N; %% Nth relative price

W_rp = diag(xi2_vecN_1); %% diagonal matrix with weigths of the rel. prices

W_ri = diag(xi3_vec); %% diagonal matrix with weights of sectoral inflation

W_mat(jump_agg+1:N+jump_agg,jump_agg+1:N+jump_agg) = W_ri; %% weights on sectoral inflation

W_mat(N+jump_agg+1:2*N-1+jump_agg,N+jump_agg+1:2*N-1+jump_agg) = W_rp;
                            %% weight on N-1 relative prices

W_mat(1,N+jump_agg+1:2*N-1+jump_agg) = -xi_rpc_vec; %% N-1 rel. prices time consumption (column)

W_mat(N+jump_agg+1:2*N-1+jump_agg,1) = -xi_rpc_vec; %% N-1 rel. prices time consumption (raw)

W_mat(1,5) = -an_vec(N)*eta_vec(N); %% N rel. price times consumption (col)

W_mat(5,1) = -an_vec(N)*eta_vec(N); %% N rel. price times consumption (raw)



W_mat(2*N-1+jump_agg+1:3*N-1+jump_agg-1,N+jump_agg+1:2*N-1+jump_agg) = -diag(xi_rpz_vec); %% N-1 rel. prices time sect. shocks (column)
      
W_mat(N+jump_agg+1:2*N-1+jump_agg,2*N-1+jump_agg+1:3*N-1+jump_agg-1) = -diag(xi_rpz_vec); %% N-1 rel. prices time sect. shocks (raw)

W_mat(3*N-1+jump_agg+1,N+jump_agg+1:2*N-1+jump_agg) = -xi_rpz_vec; %% N-1 rel. prices time common shock (column)
      
W_mat(N+jump_agg+1:2*N-1+jump_agg,3*N-1+jump_agg+1) = -xi_rpz_vec; %% N-1 rel. prices time common shock (raw)



W_mat(3*N-1+jump_agg,5) = -an_vec(N)*eta_vec(N)*czAlpha(N); %% N rel. price times N sect. shock (col)

W_mat(5,3*N-1+jump_agg) = -an_vec(N)*eta_vec(N)*czAlpha(N); %% N rel. price times N sect. shock (raw)

W_mat(3*N-1+jump_agg+1,5) = -an_vec(N)*eta_vec(N)*czAlpha(N); %% N rel. price times common shock (col)

W_mat(5,3*N-1+jump_agg+1) = -an_vec(N)*eta_vec(N)*czAlpha(N); %% N rel. price times common shock (raw)


W_mat(1,2*N-1+jump_agg+1:3*N-1+jump_agg) = xi_cz_vec; %% sector shocks time consumption (column)

W_mat(2*N-1+jump_agg+1:3*N-1+jump_agg,1) = xi_cz_vec; %% sector shocks time consumption (raw)

W_mat(1,3*N-1+jump_agg+1) = sum(xi_cz_vec); %% sector shocks time consumption (column)

W_mat(3*N-1+jump_agg+1,1) = sum(xi_cz_vec); %% sector shocks time consumption (raw)

%disp('end')