


%% calculates the matrices for REE solution. This is the version with
%% heterogeneous elasticity of substitutions and cost push shocks


function [A, B, C, kAlphaR, kAlphaC, kAlphaZ, dimK] =...
    param_natural_target7_cs(N, sigma,  beta, gamma, theta_vec, eta,...
    alpha_vec, lambda_vec, zetaZAlpha, opt_weights,an_vec,rho_c,rho_cts);

%disp('start')
%global A B C

%% NOTE: 1. opt_weights defines a vector of size N including the 
%%          inflation and output optimal weights                                              
%%       2. an_vec defines the expenditure shares from the model
%%       3. lambda_vec defines the curvature in the sector-specific
%%          production function
%%       4. the TARGETING RULE responds to the consumption-gap.
%%       5. the process for nominal income is exogenous
%%       The Consumption variable in the model is consumption in deviations from its
%%       deterministic trend.
%%       6. No weighting matrix for welfare evaluation









%% Consumption equation

nom_inc = 1; %% set ==1 if set nominal income as exogenous process




%% Define Phillips curve coefficients

eta_vec = lambda_vec.*(1+eta)-1; %% convolution of parameters...

czAlpha = -(eta_vec+1)./eta_vec; %% shocks' coefficients


den_vec = 1.+(theta_vec.*eta_vec); %% this is  the vector 1+theta(n)*eta in the
                               %% Phillips curve. It expresses the common
                               %% denominator of the variables'
                               %% coefficients

el_mc_vec = ((1-alpha_vec.*beta).*(1-alpha_vec))...
      ./alpha_vec; %% defines vector of the elasticity of inflation to the
                    %%% mc in each sector
                                  
                    
kAlphaR = el_mc_vec.*((1+gamma.*eta_vec)./den_vec);  %% relative prices



kAlphaC = el_mc_vec.*((sigma+eta_vec)./den_vec); %% consumption



kAlphaZ = el_mc_vec.*((eta_vec.*czAlpha)./den_vec);  %% coefficient on sector shock


%% Consumption under flexible prices

ccf = -(sum(an_vec.*((sigma+eta_vec)./(1+eta_vec))))^(-1);

ccfAlpha_vec = ccf*((an_vec.*eta_vec.*czAlpha)./(1+eta_vec)); %% individual weight on sector-specific shocks

                                          


%% [1] Make A
dimA=4*N+9; %% NOTE: make sure this nummber is consistent with the list below 

%% LIST OF EQUATIONS

%% 1. targeting rule (cons) or real consumption (computed from nom. income
%% equation)
%% 2. inflation (CPI)
%% 3. opt_weight inflation
%% 4. average shock
%% 5. Nth relative price gap
%% 6. Nominal interest rate
%% 7. N sector-specific inflation rates
%% 8. N-1 relative price gaps
%% 9. N sector-specific shocks
%% 10. lag consumption
%% 11. nominal income growth
%% 12. lag inflation
%% 13. common tech. shock
%% 14. N lagged inflation sect.


%General Red-Solds Parameters


            dimK=3*N-1+2+1+1; %number state variables
            

jump_agg = 6; %% number of aggregate jump variables before the sector 
              %% specific variables.

              
A=zeros(dimA,dimA);


if nom_inc == 0
    
%% Targeting rule (consumption)

% A(1,2*N-1+jump_agg+1:3*N-1+jump_agg) = opt_weights(1).*ccfAlpha_vec; %% consumption
%                                             %% under flexible prices (as a
%                                             %% function of sect. shocks)

%% NOT USED HERE!

else
    
%% consumption (computed from nom. income equation)    

A(1,3*N-1+jump_agg+2) = 1;  %% exogenous nominal income

end

%% Nth relative price gap


A(5,N+jump_agg+1:2*N-1+jump_agg) = -an_vec(1:N-1)./an_vec(N);


%% Nominal interest rate

A(6,1) = 1;  %% consumption

A(6,2) = sigma^(-1); %% CPI inflation


%% All Sector specific inflation rates

A(jump_agg+1:N+jump_agg, jump_agg+1:N+jump_agg)...
    =beta.*eye(N); %% future expected inflation rates

A(jump_agg+1:N+jump_agg, 2*N-1+jump_agg+1:3*N-1+jump_agg) = diag(kAlphaZ); 
                   %% sector-specific shocks
                   
A(jump_agg+1:N+jump_agg,3*N-1+jump_agg+4) = kAlphaZ; %% common shock


%% N-1 Sector specific inflation


A(jump_agg+1:N+jump_agg-1, N+jump_agg+1:2*N-1+jump_agg)...
    =-diag(kAlphaR(1:N-1)); %% N-1 relative prices






%% N-1 Relative prices

A(N+jump_agg+1:2*N-1+jump_agg,N+jump_agg+1:2*N-1+jump_agg) = eye((N-1)); %% N-1 relative prices



%% sector-specific shocks

A(2*N-1+jump_agg+1:3*N-1+jump_agg,2*N-1+jump_agg+1:3*N-1+jump_agg) = eye(N);


%% common shock

A(3*N-1+jump_agg+4,3*N-1+jump_agg+4) = 1; 


%% Weighted average of sector-specific shocks

 
A(4,2*N-1+jump_agg+1:3*N-1+jump_agg) = an_vec';



%% Exogenous nominal income growth

A(3*N-1+jump_agg+2,3*N-1+jump_agg+2) = 1;


%% Lagged consumption

A(3*N-1+jump_agg+1,3*N-1+jump_agg+1) = 1;

%% Lagged inflation

A(3*N-1+jump_agg+3,3*N-1+jump_agg+3) = 1;

%% Lagged sectoral inflation

A(3*N-1+jump_agg+5:4*N-1+jump_agg+4,3*N-1+jump_agg+5:4*N-1+jump_agg+4) = eye(N);


%% [2] Make B


dimB=dimA;

B=zeros(dimB,dimB);


%% Targeting rule

%% LIST OF EQUATIONS

%% 1. targeting rule (cons) or real consumption (computed from nom. income
%% equation)
%% 2. inflation (CPI)
%% 3. opt_weight inflation
%% 4. average shock
%% 5. Nth relative price gap
%% 6. Nominal interest rate
%% 7. N sector-specific inflation rates
%% 8. N-1 relative price gaps
%% 9. N sector-specific shocks
%% 10. lag consumption
%% 11. nominal income growth 
%% 12. lag inflation
%% 13. common tech shock
%% 14. N lagged inflation sect.


if nom_inc == 0

%% Targeting rule

% B(1,1) = opt_weights(1);   %% consumption (NOT USED HERE)

B(1,jump_agg+1:N+jump_agg) = opt_weights(2:end)'; %% sectoral inflation rates

else
    
B(1,1) = 1; %% consumption

B(1,2) = 1; %% inflation

B(1,3*N-1+jump_agg+1) = -1; %% lagged consumption


end
   

%% Inflation (CPI)

B(2,2) = 1;

B(2,jump_agg+1:N+jump_agg) = -an_vec';


%% opt weight index

B(3,3) = 1;

B(3,jump_agg+1:N+jump_agg) = -opt_weights(2:end)';


%% Weighted average of sector-specific shocks

B(4,4) = 1;

%% Nth Relative price 

B(5,5) = 1;


%% Nominal interest rate

B(6,6) = sigma^(-1); %% nominal interest rate

B(6,1) = 1;  %% consumption


%% All sector specific inflation rates


B(jump_agg+1:N+jump_agg,jump_agg+1:N+jump_agg) = eye(N); %% inflation rates


B(jump_agg+1:N+jump_agg,1) = -kAlphaC(1:N);  %% consumption 


%% Nth sector specific inflation (depends on the Nth relative price)

B(N+jump_agg,5) = kAlphaR(N); %% Nth relative price 




%% N-1 Relative prices

B(N+jump_agg+1:2*N-1+jump_agg,2) = -ones((N-1),1);  %% CPI inflation

B(N+jump_agg+1:2*N-1+jump_agg,jump_agg+1:(N-1)+jump_agg)...
                     = eye((N-1)); %% sec. spec. infl.

B(N+jump_agg+1:2*N-1+jump_agg,N+jump_agg+1:2*N-1+jump_agg)...
                    = eye((N-1));  %% lag relative price





%% Sector specific shocks

B(2*N-1+jump_agg+1:3*N-1+jump_agg,2*N-1+jump_agg+1:3*N-1+jump_agg)...
                   = diag(zetaZAlpha);  %% autocorrelation coeff.

               
%% Common tech shock

B(3*N-1+jump_agg+4,3*N-1+jump_agg+4) = rho_cts; %% common shock
                   
%% Exogenous nominal income growth

B(3*N-1+jump_agg+2,3*N-1+jump_agg+2) = rho_c;

%% lagged consumption

B(3*N-1+jump_agg+1,1) = 1; %% consumption (this is an identity equation)

%% lagged inflation

B(3*N-1+jump_agg+3,2) = 1; %% inflation (this is an identity equation)

%% Lagged sectoral inflation

B(3*N-1+jump_agg+5:4*N-1+jump_agg+4,jump_agg+1:N+jump_agg) = eye(N); %% sect. inflation (this is an identity equation)



%% [3] Make C
dimC = [dimA N+2];  %% sector-specific shocks (N) 

C = zeros(dimC);



C(2*N-1+jump_agg+1:3*N-1+jump_agg,1:N) = eye(N);

%% Exogenous nominal income

C(3*N-1+jump_agg+2,N+1) = 1;

%% common tech shock
C(3*N-1+jump_agg+4,N+2) = 1;


                               
                                  
